#ifndef __DOF4_HH
#define __DOF4_HH

#include <mimas/object.h>

/** Parameters of 4-DOF transform.
    @see units */
class dof4: public mimas::object
{
public:
  /** Constructor.
      @param _x x-shift.
      @param _y y-shift.
      @param _z z-shift.
      @param _angle Rotation in radian. */
  dof4( double _x, double _y, int _z, double _angle ):
    x( _x ), y( _y ), z( _z ), angle( _angle ) {}
  /// Get x-shift.
  double getX(void) const { return x; }
  /// Get y-shift.
  double getY(void) const { return y; }
  /// Get z-shift (depth).
  int getZ(void) const { return z; }
  /// Get rotation in radian.
  double getAngle(void) const { return angle; }
protected:
  /// x-shift.
  double x;
  /// y-shift.
  double y;
  /// z-shift (depth).
  int z;
  /// Rotation (in radian).
  double angle;
};

#endif
