#ifndef __MI_OBJECT_TYPE_H
#define __MI_OBJECT_TYPE_H

#include <boost/smart_ptr.hpp>
#include <mimas/object.h>
#include <mimas/xml_reference.h>
#include "mi_pose.hh"

/** @addtogroup public
    @{ */
/** Type of micro-object.
    This class holds the offset for the customisable coordinate-system.
    @author Jan Wedekind (j.wedekind@shu.ac.uk) */
class mi_object_type: public mimas::object
{
 public:
  /** Constructor.
      @param configuration top-element of XML-subtree to configure this object
      from. */
  mi_object_type( const mimas::xml_reference< XalanElement >
                  &configuration );
  // Get offset.
  // const mi_pose &get_offset(void) const { return offset; }
  // Set offset.
  // void set_offset( const mi_pose &_offset ) { offset = _offset; }
  /// Name of recognition-method.
  const std::string get_recognition_method(void) const
    { return recognition; }
  /// Name of tracking-method.
  const std::string get_tracking_method(void) const
    { return tracking; }
 protected:
  // Offset.
  // mi_pose offset;
  /// Name of recognition-method to be used.
  std::string recognition;
  /// Name of tracking-method to be used.
  std::string tracking;
};

/// Pointer to object of type mi_object_type.
typedef boost::shared_ptr< mi_object_type > mi_object_type_ptr;

///@}

#endif
