#ifndef __HOUGH_TABLE_HH
#define __HOUGH_TABLE_HH

#include <boost/multi_array.hpp>
#include <map>
#include <mimas/object.h>
#include <mimas/xml_reference.h>
#include <vector>
#include "calibration.hh"

///
class hough_table: public mimas::object
{
public:
  ///
  hough_table( const calibration_ptr &_calib,
               const mimas::xml_reference< XalanElement >
               &boundedHoughNode, int _samplingRate, int _fringe,
               short int minGradient, bool _verbose );
  ///
  std::pair< mi_pose, double > recognise
  ( const std::vector< boost::numeric::ublas::vector< int > > &points,
    const mi_pose &pose );
  ///
  double get_z_resolution(void) const { return z_resolution; }
  ///
  double get_threshold(void) const { return threshold; }
protected:
  /// Calibration data.
  calibration_ptr calib;
  /// Sampling-rate.
  int sampling_rate;
  /// Fringe.
  int fringe;
  ///
  int skip;
  ///
  double z_resolution;
  ///
  double threshold;
  ///
  int max_num_features;
  ///
  std::vector< boost::multi_array< std::map< int, float >, 2 > > hough_data;
  ///
  std::vector< mi_pose > change;
  ///
  int width;
  ///
  int height;
  ///
  bool verbose;
};

typedef boost::shared_ptr< hough_table > hough_table_ptr;

#endif
