#ifndef __GEOMETRIC_MAP_HH
#define __GEOMETRIC_MAP_HH

#include <boost/numeric/ublas/vector.hpp>
#include <boost/smart_ptr.hpp>
#include <mimas/object.h>
#include <vector>
#include "mi_pose.hh"

/** Geometric hash-function and related functionality.
    This abstract class encapsulates the selected hash-map and related
    functions.
    @see geometric_map_xyz
    @see geometric_map_xyza */
class geometric_map: public mimas::object
{
 public:
  /// Constructor.
  geometric_map( const std::string &_hash_type ): hash_type(_hash_type) {}
  /** Generate exhaustive set of affine transformations.
      This function has to generate the complete set
      of affine transformations from the set of features (of a model-image).
      to compute the hash-table.
      @param features Features from model-image. */
  virtual std::vector< mi_pose > candidates
     ( const std::vector< boost::numeric::ublas::vector< int > > &features )
    const = 0;
  /** Pick a random pose from a set of features.
      A random affine transformation is generated from a set of features
      (of a scene image).
      @param features Features of scene-image. */
  virtual bool random_poses
     ( const std::vector< boost::numeric::ublas::vector< int > > &features,
       mi_pose &pose )
    const = 0;
  /** Type of hash-function.
      @return \c "xyz" or \c "xyza". */
  const std::string get_hash_type(void) const { return hash_type; }
 protected:
  /// "xyz" or "xyza"
  std::string hash_type;
};

///
typedef boost::shared_ptr< geometric_map > geometric_map_ptr;

#endif
