#ifndef __GEOMETRIC_MAP_XYZ_HH
#define __GEOMETRIC_MAP_XYZ_HH

#include <mimas/xml_reference.h>
#include "geometric_map.hh"
#include "tools.hh"

/** Hash-functions for doing 3-DOF recognition.
    @see geometric_voting_plugin */
class geometric_map_xyz: public geometric_map
{
 public:
  /** Constructor.
      @param configuration Access to xml-document. */
  geometric_map_xyz( mimas::xml_reference< XalanElement > &configuration,
                     bool _verbose );
  /** Generate exhaustive set of affine transformations.
      This function has to generate the complete set
      of affine transformations from the set of features (of a model-image).
      to compute the hash-table.
      @param features Features from model-image. */
  virtual std::vector< mi_pose > candidates
     ( const std::vector< boost::numeric::ublas::vector< int > > &features )
    const;
  /** Pick a random pose from a set of features.
      A random affine transformation is generated from a set of features
      (of a scene image).
      @param features Features of scene-image. */
  virtual bool random_poses
     ( const std::vector< boost::numeric::ublas::vector< int > > &features,
       mi_pose &pose )
    const;
 protected:
  /** Map point to hash-function.
      The corresponding hash-function (transform) to a point-feature is
      computed.
      @param p Point-feature. */
  mi_pose point_to_pose( const boost::numeric::ublas::vector< int > &p )
    const;
  /// Random number generator for selecting point-pairs.
  static rand_function< int > rnd;
  ///
  bool verbose;
};

///
typedef boost::shared_ptr< geometric_map_xyz > geometric_map_xyz_ptr;

#endif
