#include "geometric_map_xyza.hh"

using namespace boost::numeric;
using namespace mimas;
using namespace std;

rand_function< int > geometric_map_xyza::rnd;

geometric_map_xyza::geometric_map_xyza( xml_reference< XalanElement >
                                        &configuration, bool _verbose ):
  geometric_map( "xyza" ), verbose(_verbose)
{
  min_dist = atof( configuration.selectNode( "min_dist" ).
                   getNodeText().c_str() );
  max_dist = atof( configuration.selectNode( "max_dist" ).
                   getNodeText().c_str() );
}

vector< mi_pose > geometric_map_xyza::candidates
   ( const vector< ublas::vector< int > > &features ) const
{
  vector< mi_pose > retVal;

  // Iterate through all point pairs.
  for ( vector< ublas::vector< int > >::const_iterator p2 = features.begin();
        p2 != features.end(); p2++ ) {

    for ( vector< ublas::vector< int > >::const_iterator p1 = features.begin();
          p1 != features.end(); p1++ ) {
      
      // Filter for point-pairs with distance within a given range.
      // Convert to double first.
      const ublas::vector< double > v( *p2 - *p1 );
      // Compute length.
      double dist = norm_2( v );

      if ( dist >= min_dist && dist <= max_dist )
        retVal.push_back( point_pair_to_pose( *p1, *p2 ) );

    };
      
  };

  return retVal;
};

bool geometric_map_xyza::random_poses
   ( const vector< ublas::vector< int > > &features,
     mi_pose &pose ) const
{
  const ublas::vector< int > *p1( &features[ rnd( features.size() ) ] );
      
  const ublas::vector< int > *p2( &features[ rnd( features.size() ) ] );

  // Convert to double first.
  const ublas::vector< double > v( *p2 - *p1 );
  // Compute length.
  double dist = norm_2( v );

  bool retVal;

  if ( dist >= min_dist && dist <= max_dist ) {
    pose = point_pair_to_pose( *p1, *p2 );
    retVal = true;
  } else
    retVal = false;

  return retVal;
}

mi_pose geometric_map_xyza::point_pair_to_pose
   ( const ublas::vector< int > &p1,
     const ublas::vector< int > &p2 ) const
{
  const ublas::vector< double > v( p2 - p1 );
  const ublas::vector< double > t( 0.5 * ( p1 + p2 ) );
  return mi_pose( atan2( v[1], v[0] ), t[0], t[1] ).inv();
}
