#ifndef __GEOMETRIC_MAP_XYZA_HH
#define __GEOMETRIC_MAP_XYZA_HH

#include <mimas/xml_reference.h>
#include "geometric_map.hh"
#include "tools.hh"

/// Hash-functions for 4-dof recognition.
class geometric_map_xyza: public geometric_map
{
 public:
  /** Constructor.
      @param configuration XML configuration block to initialise from. */
  geometric_map_xyza( mimas::xml_reference< XalanElement > &configuration,
                      bool _verbose );
  /** Generate exhaustive set of hash-maps.
      @param features List of features to generate candidates from. */
  virtual std::vector< mi_pose > candidates
     ( const std::vector< boost::numeric::ublas::vector< int > > &features )
    const;
  /** Select a random set of hash-maps.
      @param features List of features to generate random candidates from. */
  virtual bool random_poses
     ( const std::vector< boost::numeric::ublas::vector< int > > &features,
       mi_pose &pose )
    const;
 protected:
  /** Convert a tupple of feature-points to a hash-map.
      @param p1 first feature-point.
      @param p2 second feature-point. */
  mi_pose point_pair_to_pose( const boost::numeric::ublas::vector< int > &p1,
                              const boost::numeric::ublas::vector< int > &p2 )
    const;
  /// Minimum distance of two points of a point-pair.
  double min_dist;
  /// Maximum distance of two points of a point-pair.
  double max_dist;
  /// Random number generator for selecting point-pairs.
  static rand_function< int > rnd;
  ///
  bool verbose;
};

///
typedef boost::shared_ptr< geometric_map_xyza > geometric_map_xyza_ptr;

#endif
