#ifndef MMVL_HOUGH_TRACKING_PLUGIN_HH
#define MMVL_HOUGH_TRACKING_PLUGIN_HH

#include "hough_tracking.hh"
#ifndef NDEBUG
#include <mimas/image_mesaoutput.h>
#endif
#include <mimas/xml_node_reference_list.h>
#include <mimas/xml_reference.h>
#include <string>
#include "units.h"
#include "recognition_method_4dof.hh"


class hough_tracking_plugin: public recognition_method_4dof
{
 public:
  /** Constructor.
      @param _calib Calibration data.
      @param configuration Top-element of vision-configuration.
      @param _verbose Set to \c true, if you want to get verbose information
      from the vision-algorithms. */
  hough_tracking_plugin( const calibration_ptr &_calib,
                         const mimas::xml_reference< XalanElement >
                         &configuration, bool _verbose );
  /** Recognise objects using hough-tracking.
      @param image Image to recognise objects in.
      @param timestamp Timestamp for image
      @param scene Scene information. */
  virtual std::vector< match > recognise
  ( const mimas::image< unsigned char > &image,
    double timestamp,
    mi_scene_ptr scene ) const;
  
protected:
  
  std::vector< std::vector< Feature> >
  create_model( int sampling_rate, int model_width, int model_height,
                  int center_x, int center_y,
                  double theta, int min_gradient,
                  mimas::xml_node_reference_list &file_names );

  //!!check on .cc that  all this
  //variables are qualified by the object name
  class Tracker{
  public:
    float thresh;
    int nb_fail; //number of successive image failed before considering the tracking is failed.
    //--------------------//
    
    int min_gradient; 
    float z_resolution;
    int sampling_rate;
    hough_tracking_ptr tracker;
  };

  std::map<std::string, Tracker> trackers;
    
#ifndef NDEBUG
  mutable mimas::x11_display dpy;
  /// Display for debugging.
  mutable mimas::image_mesaoutput< mimas::rgba< unsigned char > > display;
#endif
};

#endif
