#include <mimas/xml_node_reference_list.h>
#include "human_vision_plugin.hh"
#include "mi_recognition.hh"

using namespace boost::numeric;
using namespace mimas;
using namespace std;

#define VISIONTYPE "human-vision"

human_vision_plugin::human_vision_plugin
  ( const calibration_ptr &_calib,
    const xml_reference< XalanElement > &configuration,
    const mi_recognition *_recognition,
    bool _verbose ):
    recognition_method( _calib, Recognition, configuration, _verbose ),
    recognition( _recognition )
{
  xml_node_reference_list objects
    ( configuration.selectNodes
      ( "database/object-type/recognition/"VISIONTYPE ) );

  for ( int i=0; i<objects.getLength(); i++ ) {
    xml_reference< XalanElement > item
      ( dynamic_pointer_cast< XalanElement >( objects.item( i ) ) );
    string className( item.selectNode( "../../@name" ).getNodeText() );
    models.insert( className );
  };
}

vector< match > human_vision_plugin::recognise
  ( const image< unsigned char > &img, double timestamp,
    mi_scene_ptr scene ) const
{
  if ( verbose ) cout << "   "VISIONTYPE":" << endl;

  vector< match > retVal;

  for ( map< string, int >::const_iterator i = scene->get_jobs().begin();
        i != scene->get_jobs().end(); i++ )

    if ( models.find( i->first ) != models.end() ) {

      if ( verbose ) cout << "      object \"" << i->first << "\":" << endl;

      double angle;
      int x;
      int y;

      // At the moment only a single call is made (regardless of the value of
      // i->second).
      if ( verbose ) cout << "         Human claims successful recognition: ";

      if ( recognition->human_vision( i->first, img, timestamp,
                                      &angle, &x, &y ) ) {
        if ( verbose ) cout << "yes";
        mi_pose pose( angle, x, y );
        retVal.push_back( match( i->first,
                                 convert_pose_to_external( i->first,
                                                           pose3d( pose ) ),
                                 1.0 ) );
      } else
        if ( verbose ) cout << "no";

      if ( verbose ) cout << endl;

    };
  return retVal;
}

mi_pose human_vision_plugin::pose3d( const mi_pose &pose ) const
{
  assert( pose.getRotation().size1() == 2 );
  // It's better to check twice.
  assert( pose.getRotation().size1() == 2 );
  assert( pose.getTranslation().size() == 2 );

  ublas::matrix< double > rotation3d( 3, 3 );
  rotation3d( 0, 0 ) = pose.getRotation()( 0, 0 );
  rotation3d( 0, 1 ) = pose.getRotation()( 0, 1 );
  rotation3d( 1, 0 ) = pose.getRotation()( 1, 0 );
  rotation3d( 1, 1 ) = pose.getRotation()( 1, 1 );
  rotation3d( 2, 2 ) = 1.0;

  ublas::vector< double > translation3d( 3 );
  translation3d( 0 ) =
    ( pose.getTranslation()( 0 ) - calib->get_middle_x() ) *
    calib->get_pixel_size();
  translation3d( 1 ) =
    ( pose.getTranslation()( 1 ) - calib->get_middle_y() ) *
    calib->get_pixel_size();
  translation3d( 2 ) = 0.0;

  return mi_pose( rotation3d, translation3d );
}
