#ifndef __HUMANVISION_PLUGIN_H
#define __HUMANVISION_PLUGIN_H

#include <set>
#include "recognition_method.hh"

class mi_recognition;

/** Class for doing human vision.
    The only purpose of this class is to provide human-vision as a
    vision-method out of many.

    This plugin calls the method \c mi_recognition::human_vision.
    @see mi_recognition::human_vision */
class human_vision_plugin: public recognition_method
{
 public:
  /** Constructor.
      @param _calib Calibration data.
      @param configuration XML document with configuration.
      @param _recognition The main-recognition object.
      @param _verbose Set to \c true, if you want to get verbose information
      from the vision-algorithms. */
  human_vision_plugin( const calibration_ptr &_calib,
                       const mimas::xml_reference< XalanElement >
                       &configuration,
                       const mi_recognition *_recognition,
                       bool _verbose );
  /** Recognise objects using a human.
      @param image Image to recognise objects in.
      @param timestamp Timestamp for image
      @param scene Scene information. */
  virtual std::vector< match > recognise
  ( const mimas::image< unsigned char > &img,
    double timestamp,
    mi_scene_ptr scene ) const;
 protected:
  /** Convert two-dimensional pose to 3d-pose.
      @param pose 2D-pose.
      @return 3D-pose*/
  mi_pose pose3d( const mi_pose &pose ) const;
  /// Types of micro-objects, which are recognised using this vision-method.
  std::set< std::string > models;
  /** Pointer to recognition-method.
      This pointer is used for calling the method
      mi_recognition::human_vision */
  const mi_recognition *recognition;
};

///
typedef boost::shared_ptr< human_vision_plugin > human_vision_plugin_ptr;

#endif
