#ifndef __MAXIMA_EXTRACTION_PLUGIN_HH
#define __MAXIMA_EXTRACTION_PLUGIN_HH

#include <set>
#include "recognition_method.hh"

/** Class for extracting local maxima.
    The input-images are gauss-filtered and thresholded. Afterwards the
    maxima are returned.

    This simple algorithm is used for recognition of the solder-spheres, which
    are located on a black back-ground. */
class maxima_extraction_plugin: public recognition_method
{
 public:
  /** Constructor.
      @param _calib Calibration data.
      @param configuration Access to XML-document.
      @param _verbose Set to \c true, if you want to get verbose information
      from the vision-algorithms. */
  maxima_extraction_plugin( const calibration_ptr &_calib,
                            const mimas::xml_reference< XalanElement >
                            &configuration,
                            bool _verbose );
  /** Interpret maxima as micro-objects.
      @param image Image to recognise objects in.
      @param timestamp Timestamp for image
      @param scene Scene information. */
  virtual std::vector< match > recognise
  ( const mimas::image< unsigned char > &img,
    double timestamp,
    mi_scene_ptr scene ) const;
 protected:
  /** Convert 2D-transform to 3D-transform.
      @param pose 2D-pose. */
  mi_pose pose3d( const mi_pose &pose ) const;
  /// Micro-object types being recognised using this vision-method.
  std::set< std::string > models;
  /// Threshold for filtered images.
  int threshold;
  /// Sigma of gauss-filter.
  double sigma;
};

#endif
