#include <mimas/xml_node_reference_list.h>
#include <sstream>
#include "recognition_method.hh"

using namespace mimas;
using namespace std;

recognition_method::recognition_method
  ( const calibration_ptr &_calib,
    vision_type _type,
    const mimas::xml_reference< XalanElement >
    &configuration,
    bool _verbose ):
    calib( _calib ), type( _type ), verbose( _verbose )
{
  xml_node_reference_list objects
    ( configuration.selectNodes( "database/object-type" ) );

  for ( int i=0; i<objects.getLength(); i++ ) {

    xml_reference< XalanElement > object
      ( dynamic_pointer_cast< XalanElement >( objects.item( i ) ) );

    // Read global offset for object.
    mi_pose pose( dynamic_pointer_cast< XalanElement >
                  ( object.selectNode( "offset" ) ) );

    // Get type-name of micro-object.
    std::string type_name( object.selectNode( "@name" ).getNodeText() );

    offset[ type_name ] = pose;

  };
}

mi_pose recognition_method::convert_pose_to_external( const std::string &type,
                                                      const mi_pose &pose )
  const throw (mimasexception)
{
  MMERROR( offset.find( type ) != offset.end(), mimasexception, ,
           "No offset specified for object-type \"" << type << "\"." );
  return mi_pose::translate_3d( 0.0, 0.0, +calib->get_focus_distance() ) *
    pose * offset.find( type )->second;
}

mi_pose recognition_method::convert_pose_to_internal( const std::string &type,
                                                      const mi_pose &pose )
  const throw (mimasexception)
{
  MMERROR( offset.find( type ) != offset.end(), mimasexception, ,
           "No offset specified for object-type \"" << type << "\"." );
  return mi_pose::translate_3d( 0.0, 0.0, -calib->get_focus_distance() ) *
    pose * offset.find( type )->second.inv();
}

void recognition_method::custom_offset( const std::string &type,
                                        const mi_pose &pose )
{
  offset[ type ] = pose * offset[ type ];
}
