#include <cmath>
#include "recognition_method_4dof.hh"

using namespace boost::numeric;

dof4 recognition_method_4dof::convert_pose_to_4dof( const std::string &type,
                                                    const mi_pose &pose,
                                                    double z_resolution ) const
{
  mi_pose internalPose( convert_pose_to_internal( type, pose ) );
  int
    x = (int)( internalPose.getTranslation()[0] / calib->get_pixel_size() ) +
        calib->get_middle_x(),
    y = (int)( internalPose.getTranslation()[1] / calib->get_pixel_size() ) +
        calib->get_middle_y(),
    z = (int)round( -internalPose.getTranslation()[2] / z_resolution );
  double angle = atan2( internalPose.getRotation()( 1, 0 ),
                        internalPose.getRotation()( 0, 0 ) );
  return dof4( x, y, z, angle );
}

mi_pose recognition_method_4dof::convert_4dof_to_pose( const std::string &type,
                                                       const dof4 &dof,
                                                       double z_resolution )
  const
{
  ublas::matrix< double > rotation( ublas::identity_matrix< double >( 3 ) );
  double
    cosa = cos( dof.getAngle() ),
    sina = sin( dof.getAngle() );
  rotation( 0, 0 ) =  cosa;
  rotation( 1, 0 ) =  sina;
  rotation( 0, 1 ) = -sina;
  rotation( 1, 1 ) =  cosa;
  ublas::vector< double > translation( 3 );
  translation( 0 ) = ( dof.getX() - calib->get_middle_x() ) *
    calib->get_pixel_size();
  translation( 1 ) = ( dof.getY() - calib->get_middle_y() ) *
    calib->get_pixel_size();
  translation( 2 ) = -dof.getZ() * z_resolution;
  return convert_pose_to_external( type, mi_pose( rotation, translation ) );
}
