#ifndef __RECOGNITION_METHOD_4DOF_H
#define __RECOGNITION_METHOD_4DOF_H

#include "dof4.hh"
#include "recognition_method.hh"

/** Abstract class for 4-DOF vision-algorithm.
    The conversions to camera-coordinate system and back are done here.
    @author Jan Wedekind (j.wedekind@shu.ac.uk) */
class recognition_method_4dof: public recognition_method
{
public:
  /** Constructor.
      @param _calib Calibration parameters.
      @param method Type of vision-method (recognition/tracking).
      @param configuration Access to XML-document.
      @param _verbose Set to \c true, if you want to get verbose information
      from the vision-algorithms. */
  recognition_method_4dof( const calibration_ptr &_calib, vision_type method,
                           const mimas::xml_reference< XalanElement >
                           &configuration,
                           bool _verbose ):
    recognition_method( _calib, method, configuration, _verbose ) {}
  /** Convert transformation of micro-object to 4-DOF information object.      
      @param type Type of micro-object.
      @param pose 4-DOF transformation.
      @param z_resolution Resolution of z-stack to quantisise for. The
      resolution has to be specified each time, because it may depend on the
      micro-object. */
  dof4 convert_pose_to_4dof( const std::string &type,
                             const mi_pose &pose,
                             double z_resolution ) const;
  /** Convert 4-DOF information object to transformation.
      @param type Type of micro-object.
      @param dof 4-DOF information object.
      @param z_resolution Resolution of z-stack to quantisise for. The
      resolution has to be specified each time, because it may depend on the
      micro-object. */
  mi_pose convert_4dof_to_pose( const std::string &type,
                                const dof4 &dof, double z_resolution ) const;
};

#endif
