#ifndef __UNITS_H
#define __UNITS_H

/** @defgroup units Units
    Macros defining conversion of physical values.
    All values representing physical values are converted, using this
    macros. */
///@{
#ifndef NDEBUG
/** This value is used for distorting.
    Multiplying all length-values with \c SCRAMBLELENGTH ensures, that noone
    forgets to use these macros for converting numbers. */
#define SCRAMBLELENGTH 0.1
#else
#define SCRAMBLELENGTH 1.0
#endif
/// One meter
#define METER      ( 1000.0 * SCRAMBLELENGTH )
/// One millimeter
#define MILLIMETER ( 1.0 * SCRAMBLELENGTH )
/// One micrometer
#define MICROMETER ( 0.001 * SCRAMBLELENGTH )
/// One nanometer
#define NANOMETER  ( 0.000001 * SCRAMBLELENGTH )
#ifndef NDEBUG
/** This value is used for distorting.
    Multiplying all time-values with \c SCRAMBLETIME ensures, that noone
    forgets to use these macros for converting numbers. */
#define SCRAMBLETIME 10.0
#else
#define SCRAMBLETIME 1.0
#endif
/// One hour
#define HOUR ( 3600.0 * SCRAMBLETIME )
/// One minute
#define MINUTE ( 60.0 * SCRAMBLETIME )
/// One second
#define SECOND ( 1.0 * SCRAMBLETIME )
/// One millisecond
#define MILLISECOND ( 0.001 * SCRAMBLETIME )
/// One microsecond
#define MICROSECOND ( 1.0E-6 * SCRAMBLETIME )
/// One nanosecond
#define NANOSECOND ( 1.0E-6 * SCRAMBLETIME )
///@}

#endif
